@@ -1,4 +1,4 @@
-from typing import Callable
+from typing import Callable, Dict
 from urllib.parse import urlunsplit
 
 from .typing import ASGIFramework
@@ -54,3 +54,18 @@ class HTTPToHTTPSRedirectMiddleware:
             }
         )
         await send({"type": "websocket.http.response.body"})
+
+
+class DispatcherMiddleware:
+    def __init__(self, mounts: Dict[str, ASGIFramework]) -> None:
+        self.mounts = mounts
+
+    async def __call__(self, scope: dict, receive: Callable, send: Callable) -> None:
+        for path, app in self.mounts.items():
+            if scope["path"].startswith(path):
+                scope["path"] = scope["path"][len(path) :]
+                return await invoke_asgi(app, scope, receive, send)
+        await send(
+            {"type": "http.response.start", "status": 404, "headers": [(b"content-length", b"0")]}
+        )
+        await send({"type": "http.response.body"})
